import * as i0 from '@angular/core';
import { PLATFORM_ID, Injectable, Inject, Directive, Input, HostListener, NgModule } from '@angular/core';
import { isPlatformBrowser, DOCUMENT, CommonModule } from '@angular/common';
import * as i2 from '@angular/router';
import { ActivationEnd } from '@angular/router';

function _window() {
    return window;
}
class WindowRef {
    constructor(platformId) {
        this.platformId = platformId;
    }
    get nativeWindow() {
        if (isPlatformBrowser(this.platformId)) {
            return _window();
        }
        return {};
    }
}
/** @nocollapse */ WindowRef.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: WindowRef, deps: [{ token: PLATFORM_ID }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ WindowRef.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: WindowRef });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: WindowRef, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: Object, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }]; } });

// our root app component
/** @dynamic */
class TagCommanderService {
    constructor(winRef, router, _doc, platformId) {
        this.winRef = winRef;
        this.router = router;
        this._doc = _doc;
        this.platformId = platformId;
        this._tcContainers = [];
        this._trackRoutes = false;
        this.router.events.subscribe((_data) => {
            if (_data instanceof ActivationEnd && this._trackRoutes) {
                if (_data.snapshot.data.tcInclude !== undefined) {
                    const data = _data.snapshot.data.tcInclude;
                    data.forEach((container) => {
                        this.reloadContainer(container['ids'], container['idc'], container['options']);
                    });
                }
            }
        });
    }
    debug_logger(...args) {
        if (this.debug) {
            console.log.apply(console, args);
        }
    }
    // /**
    //  * the script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
    //  * @param {string} id the id the script node will have
    //  * @param {string} uri the source of the script
    //  * @param {string} node the node on witch the script will be placed, it can either be head or body
    // */
    addContainer(id, uri, node) {
        return new Promise((resolve) => {
            this._tcContainers.push({ id: id, uri: uri });
            const tagContainer = this._doc.createElement('script');
            tagContainer.onload = () => resolve();
            tagContainer.setAttribute('type', 'text/javascript');
            tagContainer.setAttribute('src', uri);
            tagContainer.setAttribute('id', id);
            if (typeof node !== 'string') {
                this.debug_logger('you didn\'t specify where you wanted to place the script, it will be placed in the head by default');
                this._doc.querySelector('head').appendChild(tagContainer);
            }
            else if (node.toLowerCase() === 'head' ||
                node.toLowerCase() === 'body') {
                this._doc.querySelector(node.toLowerCase()).appendChild(tagContainer);
            }
            else {
                this.debug_logger('you didn\'t correctily specify where you wanted to place the script, it will be placed in the head by default');
                this._doc.querySelector('head').appendChild(tagContainer);
            }
        });
    }
    // /**
    //  * The script URI correspond to the tag-commander script URL, it can either be a CDN URL or the path of your script
    //  * @param {string} id
    //  */
    removeContainer(id) {
        const container = this._doc.getElementById(id);
        const containers = this._tcContainers.slice(0);
        this._doc.querySelector('head').removeChild(container);
        for (let i = 0; i < containers.length; i++) {
            if (containers[i].id === id) {
                this._tcContainers.splice(i, 1);
            }
        }
    }
    // /**
    //  * will display the debug messages if true
    //  * @param {boolean} debug if set to true it will activate the debug msg default is false
    //  */
    setDebug(debug) {
        this.debug = debug;
    }
    // /**
    //  * allows the router to be tracked
    //  * @param {boolean} b will read routes if set to true
    //  */
    trackRoutes(b) {
        this._trackRoutes = !!b;
    }
    // /**
    //  * set or update the value of the var
    //  * @param {string} tcKey
    //  * @param {*} tcVar
    //  */
    setTcVar(tcKey, tcVar) {
        if (isPlatformBrowser(this.platformId)) {
            if (!this.winRef.nativeWindow.tc_vars) {
                throw new Error('[ngx-tag-commander] setTcVar failed because no window.tc_vars was found. Did you initialize it?');
            }
            this.winRef.nativeWindow.tc_vars[tcKey] = tcVar;
        }
    }
    // /**
    //  * set your varibles for the different providers, when called the first time it
    //  * instantiate the external variable
    //  * @param {object} vars
    //  */
    setTcVars(vars) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger('setTcVars', vars);
            const listOfVars = Object.keys(vars);
            for (let i = 0; i < listOfVars.length; i++) {
                this.setTcVar(listOfVars[i], vars[listOfVars[i]]);
            }
        }
    }
    // /**
    //  * get the value of the var
    //  * @param {string} tcKey
    //  */
    getTcVar(tcKey) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger('getTcVars', tcKey);
            if (this.winRef.nativeWindow.tc_vars[tcKey] === null) {
                throw new Error('[ngx-tag-commander]tc_var is undefined. Check that it\'s properly initialized');
            }
            return this.winRef.nativeWindow.tc_vars[tcKey];
        }
    }
    // /**
    //  * removes the var by specifying the key
    //  * @param {string} varKey
    //  */
    removeTcVar(varKey) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger('removeTcVars', varKey);
            delete this.winRef.nativeWindow.tc_vars[varKey];
        }
    }
    // /**
    //  * will reload all the containers
    //  * @param {object} options can contain some options in a form of an object
    //  */
    reloadAllContainers(options = {}) {
        if (isPlatformBrowser(this.platformId)) {
            this.debug_logger('Reload all containers ', options);
            if (!this.winRef.nativeWindow.tC) {
                return window.setTimeout(() => {
                    this.reloadAllContainers(options);
                }, 1000);
            }
            this.winRef.nativeWindow.tC.container.reload(options);
        }
    }
    // /**
    //  * will reload the specified container
    //  * @param {number} siteId
    //  * @param {number} idc
    //  * @param {object} options can contain some options in a form of an object
    //  */
    reloadContainer(siteId, containerId, options) {
        if (isPlatformBrowser(this.platformId)) {
            options = options || {};
            this.debug_logger('Reload container ids: ' + siteId + ' idc: ' + containerId, typeof options === 'object' ? 'with options: ' : '', options || '');
            if (!this.winRef.nativeWindow.tC) {
                return window.setTimeout(() => {
                    this.reloadContainer(siteId, containerId, options);
                }, 1000);
            }
            this.winRef.nativeWindow.tC['container_' + siteId + '_' + containerId].reload(options);
        }
    }
    // /**
    //  * will set an TagCommander event
    //  * @param {string} eventLabel the name of your event
    //  * @param {HTMLElement} element the HTMLelement on witch the event is attached
    //  * @param {object} data the data you want to transmit
    //  */
    captureEvent(eventLabel, element, data, reloadCapture = false) {
        if (isPlatformBrowser(this.platformId)) {
            if (reloadCapture !== true) {
                this.debug_logger('captureEvent', eventLabel, element, data);
                if (typeof this.winRef.nativeWindow.tC !== 'undefined') {
                    if (eventLabel in this.winRef.nativeWindow.tC.event) {
                        this.winRef.nativeWindow.tC.event[eventLabel](element, data);
                    }
                    if (!(eventLabel in this.winRef.nativeWindow.tC.event)) {
                        setTimeout(() => {
                            this.captureEvent(eventLabel, element, data, (reloadCapture = true));
                        }, 1000);
                    }
                }
            }
        }
    }
}
/** @nocollapse */ TagCommanderService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TagCommanderService, deps: [{ token: WindowRef }, { token: i2.Router }, { token: DOCUMENT }, { token: PLATFORM_ID }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ TagCommanderService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TagCommanderService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TagCommanderService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root',
                }]
        }], ctorParameters: function () { return [{ type: WindowRef }, { type: i2.Router }, { type: Document, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: Object, decorators: [{
                    type: Inject,
                    args: [PLATFORM_ID]
                }] }]; } });

class TcSetVarsDirective {
    constructor(el, tcService) {
        this.el = el;
        this.tcService = tcService;
    }
    ngAfterViewInit() {
        this.tcService.setTcVars(this.tcSetVars);
    }
}
/** @nocollapse */ TcSetVarsDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TcSetVarsDirective, deps: [{ token: i0.ElementRef }, { token: TagCommanderService }], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ TcSetVarsDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.17", type: TcSetVarsDirective, selector: "[tcSetVars]", inputs: { tcSetVars: "tcSetVars" }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TcSetVarsDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tcSetVars]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: TagCommanderService }]; }, propDecorators: { tcSetVars: [{
                type: Input,
                args: ['tcSetVars']
            }] } });

// usage [tcEvent]
class TcEventDirective {
    constructor(el, tcService) {
        this.el = el;
        this.tcService = tcService;
    }
    onClick() {
        if (typeof this.tcEvent === 'object' && (this.tcEventLabel === undefined || this.tcEventObj === undefined)) {
            try {
                this.tcService.captureEvent(this.tcEvent.eventLabel, this.el, this.tcEvent.data);
            }
            catch (e) {
                console.error(e);
            }
        }
        else {
            this.tcService.captureEvent(this.tcEventLabel, this.el, this.tcEventObj);
        }
    }
}
/** @nocollapse */ TcEventDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TcEventDirective, deps: [{ token: i0.ElementRef }, { token: TagCommanderService }], target: i0.ɵɵFactoryTarget.Directive });
/** @nocollapse */ TcEventDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.17", type: TcEventDirective, selector: "[tcEvent]", inputs: { tcEvent: "tcEvent", tcEventLabel: "tcEventLabel", tcEventObj: "tcEventObj" }, host: { listeners: { "click": "onClick()" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: TcEventDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[tcEvent]'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: TagCommanderService }]; }, propDecorators: { tcEvent: [{
                type: Input,
                args: ['tcEvent']
            }], tcEventLabel: [{
                type: Input,
                args: ['tcEventLabel']
            }], tcEventObj: [{
                type: Input,
                args: ['tcEventObj']
            }], onClick: [{
                type: HostListener,
                args: ['click']
            }] } });

class NgxTagCommanderModule {
}
/** @nocollapse */ NgxTagCommanderModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgxTagCommanderModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ NgxTagCommanderModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgxTagCommanderModule, declarations: [TcSetVarsDirective, TcEventDirective], imports: [CommonModule], exports: [TcSetVarsDirective, TcEventDirective] });
/** @nocollapse */ NgxTagCommanderModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgxTagCommanderModule, providers: [TagCommanderService], imports: [[
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.17", ngImport: i0, type: NgxTagCommanderModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [TcSetVarsDirective, TcEventDirective],
                    imports: [
                        CommonModule
                    ],
                    exports: [TcSetVarsDirective, TcEventDirective],
                    providers: [TagCommanderService]
                }]
        }] });

/*
 * Public API Surface of ngx-tag-commander
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NgxTagCommanderModule, TagCommanderService, TcEventDirective, TcSetVarsDirective, WindowRef };
//# sourceMappingURL=ngx-tag-commander.js.map
